local LB = LibStub("LibBabble-Boss-3.0")

local THE_LICH_KING = LB["The Lich King"]

local _, _, DEFILE_ICON = GetSpellInfo(73708)
local DEFILE_LINK = GetSpellLink(73708)

local addonFrame = CreateFrame('Frame')

local failstat

addonFrame:SetScript("OnEvent", function(self)
	if not DefailureDB then
		DefailureDB = {}
	end
	failstat = DefailureDB
end)
addonFrame:RegisterEvent("ADDON_LOADED")

local function iterateRaidMembers()
	local i = 0
	local n = GetNumRaidMembers()
	return function()
			i = i + 1
			if i <= n then
				local unit = "raid" .. i
				local name = UnitName(unit)
				return unit, name
			end
		end
end

local fail = LibStub("LibFail-1.0")

local combatLogFrame = CreateFrame("Frame")
combatLogFrame:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")

local defileTargetFrame = CreateFrame('Frame')

local function onCombatLogEvent(self, _, timestamp,event,sourceGUID,sourceName, sourceFlags, destGUID, destName, destFlags, spellId)
	if event == "SPELL_CAST_START" and spellId == 72762 then
		defileTargetFrame:Show()
	end
end
combatLogFrame:SetScript("OnEvent", onCombatLogEvent)

local failed = false
local currentDefileTarget
local function scanTarget(self)
	for unitId in iterateRaidMembers() do
		local target = UnitName(unitId .. "target")
		if target == THE_LICH_KING then
			local ttarget = UnitName(unitId .. "targettarget")
			if ttarget then
				currentDefileTarget = ttarget
				failed = false
				self:Hide()
			end
			return
		end
	end
end

defileTargetFrame:SetScript("OnUpdate", scanTarget)
defileTargetFrame:Hide()

local resetFrame = CreateFrame('Frame')

local RESET_TIME = 10
local function onResetUpdate(self, elapsed)
	local elapsed = self.elapsed + elapsed
	if elapsed > RESET_TIME then
		self.elapsed = 0
		currentDefileTarget = ""
		self:Hide()
	else
		self.elapsed = elapsed
	end
end
resetFrame.elapsed = 0
resetFrame:SetScript("OnUpdate", onResetUpdate)

local combatStat = {}

local function onDefile(event, who, failtype)
	if failed then return end
	if who == currentDefileTarget then
		currentDefileTarget = nil
		return
	end
	resetFrame:Show()
	resetFrame.elapsed = 0

	local statEntry = failstat[who] or 0
	failstat[who] = statEntry + 1
	local entry = combatStat[who] or 0
	combatStat[who] = entry + 1

	failed = true
	local message = (">>>>>>>>> %s messed up %s"):format(who, DEFILE_LINK)

	SendChatMessage(message, "RAID")
end

fail.RegisterCallback("Defailure", "Fail_TheLichKing_Defile", onDefile)

local wipeCheckFrame = CreateFrame('Frame')

local function onWipe()
	for k,v in pairs(combatStat) do
		SendChatMessage(("%s failed %dx at Defile"):format(k,v), "RAID")
	end
	wipe(combatStat)
end

local function checkForWipe()
    for unit in iterateRaidMembers() do
		if unit and UnitAffectingCombat(unit) then
			return false
		end
    end
    return true
end

local function keepChecking(self)
	if checkForWipe() then
		onWipe()
		self:Hide()
	end
end
wipeCheckFrame:SetScript("OnUpdate", keepChecking)
wipeCheckFrame:SetScript("OnEvent", function(self) self:Show() end)
wipeCheckFrame:RegisterEvent("PLAYER_REGEN_DISABLED")

local ldbdata = {
	type = "data source",
	icon = DEFILE_ICON,
	OnClick = function(_, msg)
			if msg == "LeftButton" then
				SendChatMessage("Defailure - Stats: ", "RAID")
				for k,v in pairs(failstat) do
					if UnitExists(k) then
						SendChatMessage(("%s failed %dx"):format(k,v), "RAID")
					end
				end
			end
		end,
	label = "Defailure",
}

LibStub("LibDataBroker-1.1"):NewDataObject("Defailure", ldbdata)